import lz4.frame as lz4frame
import unittest
import os
import sys
import struct
from multiprocessing.pool import ThreadPool

class TestLZ4Frame(unittest.TestCase):
    def test_create_and_free_compression_context(self):
        context = lz4frame.create_compression_context()
        self.assertNotEqual(context, None)

    def test_compress(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        compressed = lz4frame.compress(input_data)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_begin_update_end(self):
        context = lz4frame.create_compression_context()
        self.assertNotEqual(context, None)

        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        chunk_size = int((len(input_data)/2)+1)
        compressed = lz4frame.compress_begin(context)
        compressed += lz4frame.compress_update(context, input_data[:chunk_size])
        compressed += lz4frame.compress_update(context, input_data[chunk_size:])
        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_huge_with_size(self):
        context = lz4frame.create_compression_context()
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123" * 4096
        chunk_size = int((len(input_data)/2)+1)
        compressed = lz4frame.compress_begin(context, source_size=len(input_data))
        compressed += lz4frame.compress_update(context, input_data[:chunk_size])
        compressed += lz4frame.compress_update(context, input_data[chunk_size:])
        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_huge_without_size(self):
        context = lz4frame.create_compression_context()
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123" * 4096
        chunk_size = int((len(input_data)/2)+1)
        compressed = lz4frame.compress_begin(context)
        compressed += lz4frame.compress_update(context, input_data[:chunk_size])
        compressed += lz4frame.compress_update(context, input_data[chunk_size:])
        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_not_defaults_1(self):
        input_data = os.urandom(10 * 128 * 1024)  # Read 10 * 128kb
        compressed = lz4frame.compress(
            input_data,
            block_size=lz4frame.BLOCKSIZE_MAX256KB,
            block_mode=lz4frame.BLOCKMODE_LINKED,
            compression_level=lz4frame.COMPRESSIONLEVEL_MAX,
        )
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_not_defaults_2(self):
        input_data = os.urandom(20 * 128 * 1024)  # Read 20 * 128kb
        compressed = lz4frame.compress(
            input_data,
            block_size=lz4frame.BLOCKSIZE_MAX1MB,
            block_mode=lz4frame.BLOCKMODE_INDEPENDENT,
            compression_level=lz4frame.COMPRESSIONLEVEL_MIN,
            content_checksum=lz4frame.CONTENTCHECKSUM_DISABLED
        )
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_not_defaults_3(self):
        input_data = os.urandom(20 * 128 * 1024)  # Read 20 * 128kb
        compressed = lz4frame.compress(
            input_data,
            block_size=lz4frame.BLOCKSIZE_MAX64KB,
            block_mode=lz4frame.BLOCKMODE_LINKED,
            compression_level=lz4frame.COMPRESSIONLEVEL_MINHC,
            content_checksum=lz4frame.CONTENTCHECKSUM_DISABLED
        )
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_not_defaults_4(self):
        input_data = os.urandom(20 * 128 * 1024)  # Read 20 * 128kb
        compressed = lz4frame.compress(
            input_data,
            block_size=lz4frame.BLOCKSIZE_MAX64KB,
            block_mode=lz4frame.BLOCKMODE_LINKED,
            compression_level=lz4frame.COMPRESSIONLEVEL_MAX,
            content_checksum=lz4frame.CONTENTCHECKSUM_ENABLED
        )
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_begin_update_end_no_auto_flush(self):
        context = lz4frame.create_compression_context()
        self.assertNotEqual(context, None)

        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        chunk_size = int((len(input_data)/2)+1)
        compressed = lz4frame.compress_begin(context, auto_flush=0)
        compressed += lz4frame.compress_update(context, input_data[:chunk_size])
        compressed += lz4frame.compress_update(context, input_data[chunk_size:])
        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_begin_update_end_no_auto_flush_2(self):
        input_data = os.urandom(4 * 128 * 1024)  # Read 4 * 128kb
        context = lz4frame.create_compression_context()
        self.assertNotEqual(context, None)
        compressed = lz4frame.compress_begin(context, auto_flush=0)
        chunk_size = 32 * 1024 # 32 kb, half of default block size
        start = 0
        end = start + chunk_size

        while start <= len(input_data):
            compressed += lz4frame.compress_update(context, input_data[start:end])
            start = end
            end = start + chunk_size

        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_begin_update_end_not_defaults(self):
        input_data = os.urandom(10 * 128 * 1024)  # Read 10 * 128kb
        context = lz4frame.create_compression_context()
        self.assertNotEqual(context, None)
        compressed = lz4frame.compress_begin(
            context,
            block_size=lz4frame.BLOCKSIZE_MAX256KB,
            block_mode=lz4frame.BLOCKMODE_LINKED,
            compression_level=lz4frame.COMPRESSIONLEVEL_MINHC,
            auto_flush=1
        )
        chunk_size = 128 * 1024 # 128 kb, half of block size
        start = 0
        end = start + chunk_size

        while start <= len(input_data):
            compressed += lz4frame.compress_update(context, input_data[start:end])
            start = end
            end = start + chunk_size

        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_begin_update_end_no_auto_flush_not_defaults(self):
        input_data = os.urandom(10 * 128 * 1024)  # Read 10 * 128kb
        context = lz4frame.create_compression_context()
        self.assertNotEqual(context, None)
        compressed = lz4frame.compress_begin(
            context,
            block_size=lz4frame.BLOCKSIZE_MAX256KB,
            block_mode=lz4frame.BLOCKMODE_LINKED,
            compression_level=lz4frame.COMPRESSIONLEVEL_MAX,
            auto_flush=0
        )
        chunk_size = 128 * 1024 # 128 kb, half of block size
        start = 0
        end = start + chunk_size

        while start <= len(input_data):
            compressed += lz4frame.compress_update(context, input_data[start:end])
            start = end
            end = start + chunk_size

        compressed += lz4frame.compress_end(context)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_get_frame_info(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        compressed = lz4frame.compress(
            input_data,
            lz4frame.COMPRESSIONLEVEL_MAX,
            lz4frame.BLOCKSIZE_MAX64KB,
            lz4frame.CONTENTCHECKSUM_DISABLED,
            lz4frame.BLOCKMODE_INDEPENDENT,
            lz4frame.FRAMETYPE_FRAME
        )

        frame_info = lz4frame.get_frame_info(compressed)

        self.assertEqual(
            frame_info,
            {
                "blockSizeID":lz4frame.BLOCKSIZE_MAX64KB,
                "blockMode":lz4frame.BLOCKMODE_INDEPENDENT,
                "contentChecksumFlag":lz4frame.CONTENTCHECKSUM_DISABLED,
                "frameType":lz4frame.FRAMETYPE_FRAME,
                "contentSize":len(input_data)
            }
        )

    def test_threads(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtrip(x):
            return lz4frame.decompress(lz4frame.compress(x))

        pool = ThreadPool(8)
        out = pool.map(roundtrip, data)
        pool.close()
        self.assertEqual(data, out)

    def test_compress_begin_update_end_no_auto_flush_not_defaults_threaded(self):
        data = [os.urandom(3 * 256 * 1024) for i in range(100)]

        def roundtrip(x):
            context = lz4frame.create_compression_context()
            self.assertNotEqual(context, None)
            compressed = lz4frame.compress_begin(
                context,
                block_size=lz4frame.BLOCKSIZE_MAX256KB,
                block_mode=lz4frame.BLOCKMODE_LINKED,
                compression_level=lz4frame.COMPRESSIONLEVEL_MAX,
                auto_flush=0
            )
            chunk_size = 128 * 1024 # 128 kb, half of block size
            start = 0
            end = start + chunk_size

            while start <= len(x):
                compressed += lz4frame.compress_update(context, x[start:end])
                start = end
                end = start + chunk_size

            compressed += lz4frame.compress_end(context)
            decompressed = lz4frame.decompress(compressed)
            return decompressed

        pool = ThreadPool(8)
        out = pool.map(roundtrip, data)
        pool.close()
        self.assertEqual(data, out)

    def test_LZ4FrameCompressor(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        with lz4frame.LZ4FrameCompressor() as compressor:
            compressed = compressor.compress_begin()
            compressed += compressor.compress(input_data)
            compressed += compressor.flush()
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_LZ4FrameCompressor_reset(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        with lz4frame.LZ4FrameCompressor() as compressor:
            compressed = compressor.compress_begin()
            compressed += compressor.compress(input_data)
            compressed += compressor.flush()
            compressor.reset()
            compressed = compressor.compress_begin()
            compressed += compressor.compress(input_data)
            compressed += compressor.flush()
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

    def test_compress_without_content_size(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        compressed = lz4frame.compress(input_data, content_size_header=False)
        frame = lz4frame.get_frame_info(compressed)
        self.assertEqual(frame['contentSize'], 0)
        decompressed = lz4frame.decompress(compressed)
        self.assertEqual(input_data, decompressed)

class TestLZ4FrameModern(unittest.TestCase):
    def test_decompress_truncated(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        for chksum in (lz4frame.CONTENTCHECKSUM_DISABLED, lz4frame.CONTENTCHECKSUM_ENABLED):
            for conlen in (0, len(input_data)):
                context = lz4frame.create_compression_context()
                compressed = lz4frame.compress_begin(context, content_checksum=chksum, source_size=conlen)
                compressed += lz4frame.compress_update(context, input_data)
                compressed += lz4frame.compress_end(context)
                for i in range(len(compressed)):
                    with self.assertRaisesRegexp(RuntimeError, r'^(LZ4F_getFrameInfo failed with code: ERROR_frameHeader_incomplete|LZ4F_freeDecompressionContext reported unclean decompressor state \(truncated frame\?\): \d+)$'):
                        lz4frame.decompress(compressed[:i])

    def test_checksum_failure(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        compressed = lz4frame.compress(input_data, content_checksum=lz4frame.CONTENTCHECKSUM_ENABLED)
        with self.assertRaisesRegexp(RuntimeError, r'^LZ4F_decompress failed with code: ERROR_contentChecksum_invalid'):
            last = struct.unpack('B', compressed[-1:])[0]
            lz4frame.decompress(compressed[:-1] + struct.pack('B', last ^ 0x42))
        # NB: blockChecksumFlag is not supported by lz4 at the moment, so some
        # random 1-bit modifications of input may actually trigger valid output
        # without errors. And content checksum remains the same!

    def test_decompress_trailer(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        compressed = lz4frame.compress(input_data)
        with self.assertRaisesRegexp(ValueError, r'^Extra data: 64 trailing bytes'):
            lz4frame.decompress(compressed + b'A'*64)
        # This API does not support frame concatenation!
        with self.assertRaisesRegexp(ValueError, r'^Extra data: \d+ trailing bytes'):
            lz4frame.decompress(compressed + compressed)

    def test_LZ4FrameCompressor_fails(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123"
        with self.assertRaisesRegexp(RuntimeError, r'compress called after flush'):
            with lz4frame.LZ4FrameCompressor() as compressor:
                compressed = compressor.compress_begin()
                compressed += compressor.compress(input_data)
                compressed += compressor.flush()
                compressed = compressor.compress(input_data)


if sys.version_info < (2, 7):
    # Poor-man unittest.TestCase.skip for Python 2.6
    del TestLZ4FrameModern

if __name__ == '__main__':
    unittest.main()



import lz4.frame as lz4frame

context = lz4frame.create_compression_context()
input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123" * 4096 * 1000
chunk_size = int((len(input_data)/2)+1)
compressed = lz4frame.compress_begin(context, source_size=len(input_data))
compressed += lz4frame.compress_update(context, input_data[:chunk_size])
compressed += lz4frame.compress_update(context, input_data[chunk_size:])
compressed += lz4frame.compress_end(context)

with open('t.lz4', 'w') as file:
    file.write(compressed)



import lz4.block
import sys


from multiprocessing.pool import ThreadPool
import unittest
import os

class TestLZ4Block(unittest.TestCase):

    def test_empty_string(self):
      DATA = b''
      self.assertEqual(DATA, lz4.block.decompress(lz4.block.compress(DATA)))

    def test_random(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(lz4.block.compress(DATA)))

    def test_random_hc1(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(
          lz4.block.compress(DATA, mode='high_compression')))

    def test_random_hc2(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(
          lz4.block.compress(DATA, mode='high_compression', compression=7)))

    def test_random_hc3(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(
          lz4.block.compress(DATA, mode='high_compression', compression=16)))

    def test_random_fast1(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(
          lz4.block.compress(DATA, mode='fast')))

    def test_random_fast2(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(
          lz4.block.compress(DATA, mode='fast', acceleration=5)))

    def test_random_fast3(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb
      self.assertEqual(DATA, lz4.block.decompress(
          lz4.block.compress(DATA, mode='fast', acceleration=9)))

    def test_random_no_store_size(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb

      self.assertEqual(
          DATA, lz4.block.decompress(
              lz4.block.compress(DATA, store_size=False),
              uncompressed_size=128 * 1024
          )
      )

    def test_random_hc_no_store_size(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb

      self.assertEqual(
          DATA, lz4.block.decompress(
              lz4.block.compress(DATA, mode='high_compression', compression=9, store_size=False),
              uncompressed_size=128 * 1024
          )
      )

    def test_random_fast_no_store_size(self):
      DATA = os.urandom(128 * 1024)  # Read 128kb

      self.assertEqual(
          DATA, lz4.block.decompress(
              lz4.block.compress(DATA, mode='fast', acceleration=9, store_size=False),
              uncompressed_size=128 * 1024
          )
      )

    def test_threads(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtrip(x):
            return lz4.block.decompress(lz4.block.compress(x))

        pool = ThreadPool(8)
        out = pool.map(roundtrip, data)
        assert data == out
        pool.close()

    def test_threads_hc1(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtriphc(x):
            return lz4.block.decompress(lz4.block.compress(x, mode='high_compression'))

        pool = ThreadPool(8)
        out = pool.map(roundtriphc, data)
        assert data == out
        pool.close()

    def test_threads_hc2(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtriphc(x):
            return lz4.block.decompress(
                lz4.block.compress(x, mode='high_compression', compression=4))

        pool = ThreadPool(8)
        out = pool.map(roundtriphc, data)
        assert data == out
        pool.close()

    def test_threads_hc3(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtriphc(x):
            return lz4.block.decompress(
                lz4.block.compress(x, mode='high_compression', compression=9))

        pool = ThreadPool(8)
        out = pool.map(roundtriphc, data)
        assert data == out
        pool.close()

    def test_threads_fast1(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtripfast(x):
            return lz4.block.decompress(lz4.block.compress(x, mode='fast', acceleration=1))

        pool = ThreadPool(8)
        out = pool.map(roundtripfast, data)
        assert data == out
        pool.close()

    def test_threads_fast2(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtripfast(x):
            return lz4.block.decompress(lz4.block.compress(x, mode='fast', acceleration=4))

        pool = ThreadPool(8)
        out = pool.map(roundtripfast, data)
        assert data == out
        pool.close()

    def test_threads_fast3(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtripfast(x):
            return lz4.block.decompress(lz4.block.compress(x, mode='fast', acceleration=8))

        pool = ThreadPool(8)
        out = pool.map(roundtripfast, data)
        assert data == out
        pool.close()

    def test_threads_fast_no_store_size(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtripfast(x):
            return lz4.block.decompress(
                lz4.block.compress(x, mode='fast', acceleration=8, store_size=False),
                uncompressed_size=128 * 1024
            )

        pool = ThreadPool(8)
        out = pool.map(roundtripfast, data)
        assert data == out
        pool.close()

    def test_threads_hc_no_store_size(self):
        data = [os.urandom(128 * 1024) for i in range(100)]
        def roundtriphc(x):
            return lz4.block.decompress(
                lz4.block.compress(x, mode='high_compression', store_size=False, compression=4),
                uncompressed_size=128 * 1024
            )

        pool = ThreadPool(8)
        out = pool.map(roundtriphc, data)
        assert data == out
        pool.close()

    def test_block_format(self):
        data = lz4.block.compress(b'A' * 64)
        self.assertEqual(data[:4], b'\x40\0\0\0')
        self.assertEqual(lz4.block.decompress(data[4:], uncompressed_size=64), b'A' * 64)

class TestLZ4BlockModern(unittest.TestCase):
    def test_decompress_ui32_overflow(self):
        data = lz4.block.compress(b'A' * 64)
        with self.assertRaises(OverflowError):
            lz4.block.decompress(data[4:], uncompressed_size=((1<<32) + 64))

    def test_decompress_without_leak(self):
        # Verify that hand-crafted packet does not leak uninitialized(?) memory.
        data = lz4.block.compress(b'A' * 64)
        with self.assertRaisesRegexp(ValueError, r'^Decompressor wrote 64 bytes, but 79 bytes expected from header$'):
            lz4.block.decompress(b'\x4f' + data[1:])
        with self.assertRaisesRegexp(ValueError, r'^Decompressor wrote 64 bytes, but 79 bytes expected from header$'):
            lz4.block.decompress(data[4:], uncompressed_size=79)

    def test_decompress_truncated(self):
        input_data = b"2099023098234882923049823094823094898239230982349081231290381209380981203981209381238901283098908123109238098123" * 24
        compressed = lz4.block.compress(input_data)
        for i in range(len(compressed)):
            with self.assertRaisesRegexp(ValueError, '^(Input source data size too small|Corrupt input at byte \d+|Decompressor wrote \d+ bytes, but \d+ bytes expected from header)'):
                lz4.block.decompress(compressed[:i])

    def test_decompress_with_trailer(self):
        data = b'A' * 64
        comp = lz4.block.compress(data)
        with self.assertRaisesRegexp(ValueError, r'^Corrupt input at byte'):
            self.assertEqual(data, lz4.block.decompress(comp + b'A'))
        with self.assertRaisesRegexp(ValueError, r'^Corrupt input at byte'):
            self.assertEqual(data, lz4.block.decompress(comp + comp))
        with self.assertRaisesRegexp(ValueError, r'^Corrupt input at byte'):
            self.assertEqual(data, lz4.block.decompress(comp + comp[4:]))

if sys.version_info < (2, 7):
    # Poor-man unittest.TestCase.skip for Python 2.6
    del TestLZ4BlockModern


class TestLZ4BlockBufferObjects(unittest.TestCase):

    def test_bytearray(self):
        DATA = os.urandom(128 * 1024)  # Read 128kb
        compressed = lz4.block.compress(DATA)
        self.assertEqual(lz4.block.compress(bytearray(DATA)), compressed)
        self.assertEqual(lz4.block.decompress(bytearray(compressed)), DATA)

    def test_return_bytearray(self):
        if sys.version_info < (3,):
            return  # skip
        DATA = os.urandom(128 * 1024)  # Read 128kb
        compressed = lz4.block.compress(DATA)
        b = lz4.block.compress(DATA, return_bytearray=True)
        self.assertEqual(type(b), bytearray)
        self.assertEqual(bytes(b), compressed)
        b = lz4.block.decompress(compressed, return_bytearray=True)
        self.assertEqual(type(b), bytearray)
        self.assertEqual(bytes(b), DATA)

    def test_memoryview(self):
        if sys.version_info < (2, 7):
            return  # skip
        DATA = os.urandom(128 * 1024)  # Read 128kb
        compressed = lz4.block.compress(DATA)
        self.assertEqual(lz4.block.compress(memoryview(DATA)), compressed)
        self.assertEqual(lz4.block.decompress(memoryview(compressed)), DATA)

    def test_unicode(self):
        if sys.version_info < (3,):
            return  # skip
        DATA = b'x'
        self.assertRaises(TypeError, lz4.block.compress, DATA.decode('latin1'))
        self.assertRaises(TypeError, lz4.block.decompress,
                          lz4.block.compress(DATA).decode('latin1'))


if __name__ == '__main__':
    unittest.main()



from ._frame import *
from ._frame import __doc__ as _doc
__doc__ = _doc


class LZ4FrameCompressor(object):
    """Create a LZ4 compressor object, which can be used to compress data
    incrementally.

    Args:
        block_size (int): Sepcifies the maximum blocksize to use.
            Options:
            - lz4.frame.BLOCKSIZE_DEFAULT or 0: the lz4 library default
            - lz4.frame.BLOCKSIZE_MAX64KB or 4: 64 kB
            - lz4.frame.BLOCKSIZE_MAX256KB or 5: 256 kB
            - lz4.frame.BLOCKSIZE_MAX1MB or 6: 1 MB
            - lz4.frame.BLOCKSIZE_MAX4MB or 7: 4 MB
            If unspecified, will default to lz4.frame.BLOCKSIZE_DEFAULT.
        block_mode (int): Specifies whether to use block-linked
            compression. Options:
            - lz4.frame.BLOCKMODE_LINKED or 0: linked mode
            - lz4.frame.BLOCKMODE_INDEPENDENT or 1: disable linked mode
            The default is lz4.frame.BLOCKMODE_LINKED.
        compression_level (int): Specifies the level of compression used.
            Values between 0-16 are valid, with 0 (default) being the
            lowest compression (0-2 are the same value), and 16 the highest.
            Values above 16 will be treated as 16.
            Values between 4-9 are recommended. 0 is the default.
            The following module constants are provided as a convenience:
            - lz4.frame.COMPRESSIONLEVEL_MIN: Minimum compression (0)
            - lz4.frame.COMPRESSIONLEVEL_MINHC: Minimum high-compression (3)
            - lz4.frame.COMPRESSIONLEVEL_MAX: Maximum compression (16)
        content_checksum (int): Specifies whether to enable checksumming of
            the payload content. Options:
            - lz4.frame.CONTENTCHECKSUM_DISABLED or 0: disables checksumming
            - lz4.frame.CONTENTCHECKSUM_ENABLED or 1: enables checksumming
            The default is CONTENTCHECKSUM_DISABLED.
        frame_type (int): Specifies whether user data can be injected between
            frames. Options:
            - lz4.frame.FRAMETYPE_FRAME or 0: disables user data injection
            - lz4.frame.FRAMETYPE_SKIPPABLEFRAME or 1: enables user data
              injection
        auto_flush (bool): When False, the LZ4 library may buffer data until a
            block is full. When True no buffering occurs, and partially full
            blocks may be returned. The default is True.
    """

    def __init__(self,
                 block_size=BLOCKSIZE_DEFAULT,
                 block_mode=BLOCKMODE_LINKED,
                 compression_level=COMPRESSIONLEVEL_MIN,
                 content_checksum=CONTENTCHECKSUM_DISABLED,
                 frame_type=FRAMETYPE_FRAME,
                 auto_flush=True):
        self.block_size = block_size
        self.block_mode = block_mode
        self.compression_level = compression_level
        self.content_checksum = content_checksum
        self.frame_type = frame_type
        self.auto_flush = auto_flush
        self._context = create_compression_context()
        self._started = False

    def __enter__(self):
        # All necessary initialization is done in __init__
        return self

    def __exit__(self, exception_type, exception, traceback):
        # The compression context is created with an appropriate destructor, so
        # no need to del it here
        pass

    def compress_begin(self, source_size=0):
        """Begin a compression frame. The returned data contains frame header
        information. The data returned from subsequent calls to ``compress()``
        should be concatenated with this header.

        Args:
            data (bytes): data to compress
            source_size (int): Optionally specify the total size of the
                uncompressed data. If specified, will be stored in the
                compressed frame header as an 8-byte field for later use
                during decompression.

        Returns:
            bytes: frame header data
        """

        if self._started is False:
            result = compress_begin(self._context,
                                    block_size=self.block_size,
                                    block_mode=self.block_mode,
                                    frame_type=self.frame_type,
                                    compression_level=self.compression_level,
                                    content_checksum=self.content_checksum,
                                    auto_flush=self.auto_flush,
                                    source_size=source_size)

            self._started = True
            return result
        else:
            raise RuntimeError('compress_begin called when not already initialized')



    def compress(self, data):
        """Compress ``data`` (a ``bytes`` object), returning a bytes object
        containing compressed data the input.

        If ``auto_flush`` has been set to ``False``, some of ``data`` may be
        buffered internally, for use in later calls to compress() and flush().

        The returned data should be concatenated with the output of any
        previous calls to ``compress()`` and a single call to
        ``compress_begin()``.

        Args:
            data (bytes): data to compress

        Returns:
            bytes: compressed data

        """
        if self._context is None:
            raise RuntimeError('compress called after flush()')

        if self._started is False:
            raise RuntimeError('compress called before compress_begin()')

        result = compress_update(self._context, data)

        return result

    def flush(self):
        """Finish the compression process, returning a bytes object containing any data
        stored in the compressor's internal buffers and a frame footer.

        To use the LZ4FrameCompressor instance after this has been called, it
        is necessary to first call the ``reset()`` method.

        Returns:
            bytes: any remaining buffered compressed data and frame footer.

        """
        result = compress_end(self._context)
        self._context = None
        self._started = False
        return result

    def reset(self):
        """Reset the LZ4FrameCompressor instance (after a call to ``flush``) allowing
        it to be re-used

        """
        self._context = create_compression_context()
        self._started = False



#!/usr/bin/env python
from setuptools import setup, find_packages, Extension
import subprocess
import os
import sys
from distutils import ccompiler

# This is the version of the bundled LZ4 library files. This variable was
# historically set by this library, so we set it for backwards compatibility
# for now. This neds removing before 1.0 release.
LZ4_VERSION = "1.7.4.2"

def pkgconfig_cmd(cmd, libname):
    try:
        pkg_config_exe = os.environ.get('PKG_CONFIG', None) or 'pkg-config'
        # poor-man's check_output (for Python 2.6 compat)
        p = subprocess.Popen([pkg_config_exe, cmd, libname], stdout=subprocess.PIPE)
        stdout, _ = p.communicate()
        res = p.wait() # communicate already waits for us so this shouldn't block
        if res != 0:
            # pkg-config failed
            return None
        return stdout.decode('utf-8')
    except OSError:
        # pkg-config not present
        return None

def library_is_installed(libname):
    ''' Check to see if we have a library called 'libname' installed.
    
    This uses pkg-config to check for existence of the library, and
    returns True if it's found, False otherwise. If pkg-config isn't found,
    False is returned. '''
    return pkgconfig_cmd('--exists', libname) is not None

def get_cflags(libname):
    return pkgconfig_cmd('--cflags', libname).split()

def get_ldflags(libname):
    return pkgconfig_cmd('--libs', libname).split()

# Check to see if we have a lz4 library installed on the system and
# use it if so. If not, we'll use the bundled library.
liblz4_found = library_is_installed('liblz4')

# Check to see if we have the py3c headers installed on the system and
# use it if so. If not, we'll use the bundled library.
py3c_found = library_is_installed('py3c')

# Set up the extension modules. If a system wide lz4 library is found, we'll
# use that. Otherwise we'll build with the bundled one. If we're building
# against the system lz4 library we don't set the compiler flags, so they'll be
# picked up from the environment. If we're building against the bundled lz4
# files, we'll set the compiler flags to be consistent with what upstream lz4
# recommends. In addition, if we're building against the bundled library files,
# we'll set LZ4_VERSION for legacy compatibility.

include_dirs = []
libraries = []

lz4version_sources = [
    'lz4/_version.c'
]

lz4block_sources = [
    'lz4/block/_block.c'
]

lz4frame_sources = [
    'lz4/frame/_frame.c'
]

if liblz4_found is True:
    extra_link_args = get_ldflags('liblz4')
else:
    include_dirs.append('lz4libs')
    extra_link_args = []
    lz4version_sources.extend(
        [
            'lz4libs/lz4.c',
        ]
    )
    lz4block_sources.extend(
        [
            'lz4libs/lz4.c',
            'lz4libs/lz4hc.c',
        ]
    )
    lz4frame_sources.extend(
        [
            'lz4libs/lz4.c',
            'lz4libs/lz4hc.c',
            'lz4libs/lz4frame.c',
            'lz4libs/xxhash.c',
        ]
    )

if py3c_found is False:
    include_dirs.append('py3c')

compiler = ccompiler.get_default_compiler()

if compiler == 'msvc':
    extra_compile_args = ['/Ot', '/Wall']
elif compiler in ('unix', 'mingw32'):
    if liblz4_found:
        extra_compile_args = get_cflags('liblz4')
    else:
        extra_compile_args = [
            '-O3',
            '-Wall',
            '-Wundef'
        ]
else:
    print('Unrecognized compiler: {0}'.format(compiler))
    sys.exit(1)

lz4version = Extension('lz4._version',
                       lz4version_sources,
                       extra_compile_args=extra_compile_args,
                       extra_link_args=extra_link_args,
                       libraries=libraries,
                       include_dirs=include_dirs,
)

lz4block = Extension('lz4.block._block',
                     lz4block_sources,
                     extra_compile_args=extra_compile_args,
                     extra_link_args=extra_link_args,
                     libraries=libraries,
                     include_dirs=include_dirs,
)

lz4frame = Extension('lz4.frame._frame',
                     lz4frame_sources,
                     extra_compile_args=extra_compile_args,
                     extra_link_args=extra_link_args,
                     libraries=libraries,
                     include_dirs=include_dirs,
)

# If we're building against the bundled lz4 libs, we can set LZ4_VERSION. This
# is present for backward compatibility, but should be removed before 10.
if liblz4_found is False:
    with open('lz4/lz4version.py', 'w+') as f:
        f.write('# File autogenerated during install.\n')
        f.write('# Do not change. Do not store in version control.\n')
        f.write('LZ4_VERSION = \'' + LZ4_VERSION + '\'\n')

# Finally call setup with the extension modules as defined above.
setup(
    name='lz4',
    use_scm_version={
        'write_to': "lz4/version.py",
    },
    setup_requires=['setuptools_scm'],
    description="LZ4 Bindings for Python",
    long_description=open('README.rst', 'r').read(),
    author='Jonathan Underwood',
    author_email='jonathan.underwood@gmail.com',
    url='https://github.com/python-lz4/python-lz4',
    packages=find_packages(),
    ext_modules=[
        lz4version,
        lz4block,
        lz4frame
    ],
    tests_require=["nose>=1.0"],
    test_suite = "nose.collector",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
